@@ -78,7 +78,7 @@ Widget::Widget(QWidget *parent):
     widget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
 
     QVBoxLayout *vbox = new QVBoxLayout(widget);
-    const int topMargin = style()->pixelMetric(QStyle::PM_LayoutTopMargin, 0, this);
+    const int topMargin = style()->pixelMetric(QStyle::PM_LayoutTopMargin, nullptr, this);
     vbox->setContentsMargins(0, topMargin, 0, 0);
     widget->setLayout(vbox);
 
@@ -88,7 +88,8 @@ Widget::Widget(QWidget *parent):
     mPrimaryCombo = new PrimaryOutputCombo(this);
     connect(mPrimaryCombo, &PrimaryOutputCombo::changed,
             this, &Widget::changed);
-    hbox->addWidget(new QLabel(i18n("Primary display:")));
+    mPrimaryLabel = new QLabel(i18n("Primary display:"));
+    hbox->addWidget(mPrimaryLabel);
     hbox->addWidget(mPrimaryCombo);
 
     hbox->addStretch();
@@ -175,7 +176,8 @@ void Widget::setConfig(const KScreen::ConfigPtr &config)
     mPrimaryCombo->setConfig(mConfig);
     mUnifyButton->setEnabled(mConfig->outputs().count() > 1);
     mScaleAllOutputsButton->setVisible(!mConfig->supportedFeatures().testFlag(KScreen::Config::Feature::PerOutputScaling));
-
+    mPrimaryCombo->setVisible(mConfig->supportedFeatures().testFlag(KScreen::Config::Feature::PrimaryDisplay));
+    mPrimaryLabel->setVisible(mConfig->supportedFeatures().testFlag(KScreen::Config::Feature::PrimaryDisplay));
 
     for (const KScreen::OutputPtr &output : mConfig->outputs()) {
         connect(output.data(), &KScreen::Output::isEnabledChanged,
@@ -289,7 +291,7 @@ void Widget::slotUnifyOutputs()
                 continue;
             }
 
-            if (base == 0) {
+            if (!base) {
                 base = output;
             }
 
@@ -463,15 +465,23 @@ void Widget::slotIdentifyOutputs(KScreen::ConfigOperation *op)
             qWarning() << "Failed to obtain root item";
             continue;
         }
-        QSize realSize;
+
+        QSize deviceSize, logicalSize;
         if (output->isHorizontal()) {
-            realSize = mode->size();
+            deviceSize = mode->size();
         } else {
-            realSize = QSize(mode->size().height(), mode->size().width());
+            deviceSize = QSize(mode->size().height(), mode->size().width());
         }
+        if (config->supportedFeatures() & KScreen::Config::Feature::PerOutputScaling) {
+            // no scale adjustment needed on Wayland
+            logicalSize = deviceSize;
+        } else {
+            logicalSize = deviceSize / devicePixelRatioF();
+        }
+
         rootObj->setProperty("outputName", Utils::outputName(output));
-        rootObj->setProperty("modeName", Utils::sizeToString(realSize));
-        view->setProperty("screenSize", QRect(output->pos(), realSize));
+        rootObj->setProperty("modeName", Utils::sizeToString(deviceSize));
+        view->setProperty("screenSize", QRect(output->pos(), logicalSize));
         mOutputIdentifiers << view;
     }
 
