@@ -23,6 +23,7 @@
 #include "device.h"
 #include "kscreenadaptor.h"
 #include "kscreen_daemon_debug.h"
+#include "osdmanager.h"
 
 #include <QTimer>
 #include <QAction>
@@ -48,10 +49,8 @@ K_PLUGIN_FACTORY_WITH_JSON(KScreenDaemonFactory,
 KScreenDaemon::KScreenDaemon(QObject* parent, const QList< QVariant >& )
  : KDEDModule(parent)
  , m_monitoredConfig(nullptr)
- , m_iteration(Generator::None)
  , m_monitoring(false)
  , m_changeCompressor(new QTimer(this))
- , m_buttonTimer(new QTimer(this))
  , m_saveTimer(new QTimer(this))
  , m_lidClosedTimer(new QTimer(this))
  
@@ -95,10 +94,8 @@ void KScreenDaemon::init()
     connect(action, &QAction::triggered, [&](bool) { displayButton(); });
 
     new KScreenAdaptor(this);
-
-    m_buttonTimer->setInterval(300);
-    m_buttonTimer->setSingleShot(true);
-    connect(m_buttonTimer, &QTimer::timeout, this, &KScreenDaemon::applyGenericConfig);
+    // Initialize OSD manager to register its dbus interface
+    KScreen::OsdManager::self();
 
     m_saveTimer->setInterval(300);
     m_saveTimer->setSingleShot(true);
@@ -180,10 +177,49 @@ void KScreenDaemon::applyKnownConfig()
     doApplyConfig(config);
 }
 
+void KScreenDaemon::applyOsdAction(KScreen::OsdAction::Action action)
+{
+    switch (action) {
+    case KScreen::OsdAction::NoAction:
+        qCDebug(KSCREEN_KDED) << "OSD: no action";
+        return;
+    case KScreen::OsdAction::SwitchToInternal:
+        qCDebug(KSCREEN_KDED) << "OSD: switch to internal";
+        doApplyConfig(Generator::self()->displaySwitch(Generator::TurnOffExternal));
+        return;
+    case KScreen::OsdAction::SwitchToExternal:
+        qCDebug(KSCREEN_KDED) << "OSD: switch to external";
+        doApplyConfig(Generator::self()->displaySwitch(Generator::TurnOffEmbedded));
+        return;
+    case KScreen::OsdAction::ExtendLeft:
+        qCDebug(KSCREEN_KDED) << "OSD: extend left";
+        doApplyConfig(Generator::self()->displaySwitch(Generator::ExtendToLeft));
+        return;
+    case KScreen::OsdAction::ExtendRight:
+        qCDebug(KSCREEN_KDED) << "OSD: extend right";
+        doApplyConfig(Generator::self()->displaySwitch(Generator::ExtendToRight));
+        return;
+    case KScreen::OsdAction::Clone:
+        qCDebug(KSCREEN_KDED) << "OSD: clone";
+        doApplyConfig(Generator::self()->displaySwitch(Generator::Clone));
+        return;
+    }
+
+    Q_UNREACHABLE();
+}
+
 void KScreenDaemon::applyIdealConfig()
 {
-    qCDebug(KSCREEN_KDED) << "Applying ideal config";
-    doApplyConfig(Generator::self()->idealConfig(m_monitoredConfig));
+
+    if (m_monitoredConfig->connectedOutputs().count() < 2) {
+        KScreen::OsdManager::self()->hideOsd();
+        doApplyConfig(Generator::self()->idealConfig(m_monitoredConfig));
+    } else {
+        qCDebug(KSCREEN_KDED) << "Getting ideal config from user via OSD...";
+        auto action = KScreen::OsdManager::self()->showActionSelector();
+        connect(action, &KScreen::OsdAction::selected,
+                this, &KScreenDaemon::applyOsdAction);
+    }
 }
 
 void logConfig(const KScreen::ConfigPtr &config) {
@@ -243,40 +279,18 @@ void KScreenDaemon::showOsd(const QString &icon, const QString &text)
     QDBusConnection::sessionBus().asyncCall(msg);
 }
 
+void KScreenDaemon::showOutputIdentifier()
+{
+    KScreen::OsdManager::self()->showOutputIdentifiers();
+}
+
 void KScreenDaemon::displayButton()
 {
     qCDebug(KSCREEN_KDED) << "displayBtn triggered";
 
-    QString message = i18nc("OSD text after XF86Display button press", "No External Display");
-    if (m_monitoredConfig && m_monitoredConfig->connectedOutputs().count() > 1) {
-        message = i18nc("OSD text after XF86Display button press", "Changing Screen Layout");
-    }
-    showOsd(QStringLiteral("preferences-desktop-display-randr"), message);
-
-    if (m_buttonTimer->isActive()) {
-        qCDebug(KSCREEN_KDED) << "Too fast, cowboy";
-        return;
-    }
-
-    m_buttonTimer->start();
-}
-
-void KScreenDaemon::resetDisplaySwitch()
-{
-    qCDebug(KSCREEN_KDED) << "resetDisplaySwitch()";
-    m_iteration = Generator::None;
-}
-
-void KScreenDaemon::applyGenericConfig()
-{
-    if (m_iteration == Generator::ExtendToRight) {
-        m_iteration = Generator::None;
-    }
-
-    m_iteration = Generator::DisplaySwitchAction(static_cast<int>(m_iteration) + 1);
-    qCDebug(KSCREEN_KDED) << "displayButton: " << m_iteration;
-
-    doApplyConfig(Generator::self()->displaySwitch(m_iteration));
+    auto action = KScreen::OsdManager::self()->showActionSelector();
+    connect(action, &KScreen::OsdAction::selected,
+            this, &KScreenDaemon::applyOsdAction);
 }
 
 void KScreenDaemon::lidClosedChanged(bool lidIsClosed)
@@ -347,8 +361,6 @@ void KScreenDaemon::outputConnectedChanged()
         m_changeCompressor->start();
     }
 
-    resetDisplaySwitch();
-
     KScreen::Output *output = qobject_cast<KScreen::Output*>(sender());
     qCDebug(KSCREEN_KDED) << "outputConnectedChanged():" << output->name();
 
@@ -436,5 +448,4 @@ KScreen::OutputPtr KScreenDaemon::findEmbeddedOutput(const KScreen::ConfigPtr &c
 }
 
 
-
 #include "daemon.moc"
